<?php

	/*  Copyright 2007 J.P.Jarolim (email : yapb@johannes.jarolim.com)

		This program is free software; you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation; either version 2 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program; if not, write to the Free Software
		Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	*/

	class Yapb {

		/**
		 * Current plugin version
		 * @var string
		 **/
		var $pluginVersion;

		/**
		 * Lowest required WordPress Version
		 * @var string
		 **/
		var $requiredWordPressVersion;

		/**
		 * Highest WordPress Version this plugin version was tested with
		 * @var string
		 **/
		var $highestTestedWordPressVersion;

		/**
		 * The templating engine instance
		 * @var Savant2
		 */
		var $tpl = null;
		
		/**
		 * @var string
		 */
		var $base_url = '';
		
		/**
		 * The directory separator of this os
		 * @var string
		 */
		var $separator;

		/**
		 * The array holding all configuration settings and defaults for YAPB
		 * @var array
		 */
		var $options;

		/**
		 * PHP4 Constructor
		 */
		function Yapb() {	
			$this->__construct(); 
		}

		/**
		 * PHP5 Constructor
		 */
		function __construct() {
		
			global $wpdb;

			// First of all, let's read some version information
			$this->readVersionInformation();

			// Now, let's define some globally available variables
			require_once realpath(dirname(__FILE__) . '/../includes/YapbConstants.script.php');

			// I18N support through GNU-Gettext files
			load_plugin_textdomain('yapb', 'wp-content/plugins/' . YAPB_PLUGINDIR_NAME . '/lang/');

			// Let's require some usefull stuff
			require_once realpath(dirname(__FILE__) . '/YapbImage.class.php');
			require_once realpath(dirname(__FILE__) . '/YapbUtils.class.php');
			require_once realpath(dirname(__FILE__) . '/ExifUtils.class.php');
			require_once realpath(dirname(__FILE__) . '/Savant2-2.4.3/Savant2.php');
			require_once realpath(dirname(__FILE__) . '/../includes/YapbTemplateFunctions.php');

			// Initialize the savant2 templating engine
			$this->tpl =& new Savant2();
			$this->tpl->addPath('template', YAPB_TPL_PATH);

			// Log4php initialization
			require_once realpath(dirname(__FILE__) . '/log4php-0.9/src/log4php/LoggerManager.php');
			$log = LoggerManager::getLogger(__FILE__);
			$log->debug('## YAPB START');
			
			// Dashboard Activity Box
			add_action('activity_box_end', array(&$this, 'activity_box_end'));
			
			// wp-admin post.php hooks
			add_filter('edit_form_advanced', array(&$this, 'edit_form_advanced'));
			if (get_settings('yapb_form_on_page_form')) {
				add_filter('edit_page_form', array(&$this, 'edit_form_advanced'));
			}
			
			add_action('edit_post', array(&$this, 'edit_publish_save_post'));

			add_action('publish_post', array(&$this, 'edit_publish_save_post'));
			add_action('publish_post', array(&$this, 'publish_post'));
			add_action('save_post', array(&$this, 'edit_publish_save_post'));
			add_action('delete_post', array(&$this, 'delete_post'));
			
			// wp-admin edit.php hooks
			add_filter('manage_posts_columns', array(&$this, 'manage_posts_columns'));
			add_action('manage_posts_custom_column', array(&$this, 'manage_posts_custom_column'));

			// the wp-loop
			add_filter('the_posts', array(&$this, 'the_posts'));

			// options
			add_action('admin_menu', array(&$this, 'add_options_panel'));

			// Activation, Deactivation

			register_activation_hook(YAPB_PLUGINDIR_NAME . '/Yapb.php', array(&$this, 'activate_pluginurl'));
			register_deactivation_hook(YAPB_PLUGINDIR_NAME . '/Yapb.php', array(&$this, 'deactivate_pluginurl'));

			// Feeds & automatic image insertion
			add_filter('the_content', array(@$this, 'the_content'));

			// We fill the options array
			$this->options = array(
				// OptionGroup(title, description)
				array(__('Yet Another Photoblog Options', 'yapb'), '',
					// OptionSubGroups Array
					array(
						array(__('Writing Options', 'yapb'), __('These settings do alter the behaviour of the WordPress input mask for new articles.', 'yapb'),
							// Option Array
							array(
								// option_name => Option(type, text, [SELECT:optionfields,] defaultvalue)
								'yapb_check_post_date_from_exif' => array('CHECKBOX', __('Check by default: Post date from image exif data if available.', 'yapb'), true),
								'yapb_default_post_category' => array('CHECKBOX_SELECT', __('Assign post exclusivly to category # if attaching an YAPB-image.', 'yapb'), $this->_options_categories_array(), array(false, '')),
								'yapb_form_on_page_form' => array('CHECKBOX', __('Enable YAPB-Imageupload for content pages', 'yapb'), false)
							)
						),
/* 						array(__('Image Resizing on upload','yapb'), '',
							array(
								'yapb_resize_on_upload_activate' => array('CHECKBOX', __('Enable Image Resizing on upload', 'yapb'), false),
								'yapb_resize_on_upload_max_dimesion' => array('INPUT', __('Maximum image side length: #5 px', 'yapb'), '1024')
							)
						),  */
						array(__('EXIF Filtering Options', 'yapb'), '',
							array(
								'yapb_filter_exif_data' => array('CHECKBOX', __('Enable EXIF tags filtering', 'yapb'), false),
								'yapb_view_exif_tagnames' => array('CUSTOM_VIEW_EXIF_TAGNAMES', __('Show the following EXIF tags if available:<br /><i><small>(This list will be filled up after you uploaded your first image)</small></i>', 'yapb'), array())
							)
						),
						array('Update Services', '',
							array(
								'yapb_ping_sites' => array('TEXTAREA', __('YAPB notifies the following site update services if you publish a photoblog-post.<br />These services will be pinged additionally to the services defined on the options/write admin-panel.<br />Separate multiple service URIs with line breaks.', 'yapb'), '')
							)
						),
						array('Feeds', __('YAPB may embed images/thumbnails in your RSS2 and ATOM feeds. You will have to turn on this feature if you want to subscribe to services like <a href="http://photos.vfxy.com" target="_blank">VFXY</a>.', 'yapb'),
							array(
								'yapb_display_images_xml' => array('CHECKBOX', __('<strong>Embed images in RSS2 and ATOM feeds content.</strong>', 'yapb'), true),
								'yapb_display_images_xml_thumbnail' => array('CHECKBOX_INPUT', __('Display as thumbnail with a width of #3 px', 'yapb'), array(true, '180')),
								'yapb_display_images_xml_inline_style' => array('INPUT', __('Inline CSS-Style for image tag: #40', 'yapb'), 'float:left;padding:0 10px 10px 0;'),
								'yapb_display_images_xml_html_before' => array('TEXTAREA', __('Custom HTML before image tag', 'yapb'), ''),
								'yapb_display_images_xml_html_after' => array('TEXTAREA', __('Custom HTML after image tag', 'yapb'), '')
							)
						)
					)
				),
				
				array(__('Thumbnailer Library Options', 'yapb'), __('<a href="http://phpthumb.sourceforge.net/" target="_blank">phpThumb</a> is the thumbnailing library of my choice. For your comfort, i made available a selection of settings. For more Information refer to <a href="http://phpthumb.sourceforge.net/" target="_blank">http://phpthumb.sourceforge.net/</a>.', 'yapb'),
					array(
						array(__('ImageMagick configuration', 'yapb'), __('If source image is larger than available memory limits AND ImageMagick\'s "convert" program is available on your server (<a href="http://www.imagemagick.org" target="_blank">more info</a>), phpThumb() will call ImageMagick to perform the thumbnailing of the source image to bypass the memory limitation.', 'yapb'),
							array(
								'yapb_phpthumb_imagemagick_path' => array('INPUT', __('Absolute pathname to "convert": #20 Leave empty if "convert" is in the path.', 'yapb'), '')
							)
						),
						array(__('Default output configuration', 'yapb'), '',
							array(
								'yapb_phpthumb_output_format' => array('SELECT', __('Default output format: # Thumbnail will be output in this format (if available in your version of GD).', 'yapb'), array('JPG' => 'jpeg', 'PNG' => 'png', 'GIF' => 'gif'), 'JPG'),
								'yapb_phpthumb_output_interlace' => array('CHECKBOX', __('Interlaced output for GIF/PNG, progressive output for JPEG; if unchecked: non-interlaced for GIF/PNG, baseline for JPEG.', 'yapb'), false)
							)
						)
					)
				),

				array(__('Automatic Template Insertion', 'yapb'), '',
					array(
						array(__('General', 'yapb'), __('Yapb does display uploaded images automatically on different sections of your site by default.<br />This is NOT exactly what i thought of when i designed and programmed YAPB - It\'s just a help for first-time-users.<br />To style your photoblog individually, turn off this option and have a look at <a target="_blank" href="http://johannes.jarolim.com/blog/wordpress/yet-another-photoblog/adapting-templates/">how to adapt themes manually</a>.', 'yapb'),
							array(
								'yapb_display_images_activate' => array('CHECKBOX', '<strong>' . __('Activate automatic image rendering in general.', 'yapb') . '</strong>', true),
								'yapb_display_images_xhtml' => array('CHECKBOX', __('Embed XHTML compliant HTML', 'yapb'), true)
							)
						),
						array(__('Home page', 'yapb'), __('The homepage usually shows a number of previously published posts.<br />You probably want to show thumbnails only.', 'yapb'),
							array(
								'yapb_display_images_home' => array('CHECKBOX', __('<strong>Display images on HOME page listing.</strong>', 'yapb'), true),
								'yapb_display_images_home_thumbnail' => array('CHECKBOX_INPUT', __('Display as thumbnail with a width of #3 px', 'yapb'), array(true, '200')),
								'yapb_display_images_home_inline_style' => array('INPUT', __('Inline CSS-Style for image tag: #40', 'yapb'), ''),
								'yapb_display_images_home_html_before' => array('TEXTAREA', __('Custom HTML before image tag', 'yapb'), '<div style="float:left;border:10px solid silver;margin-right:10px;margin-bottom:10px;">'),
								'yapb_display_images_home_html_after' => array('TEXTAREA', __('Custom HTML after image tag', 'yapb'), '</div>')
							)
						),
						array(__('Single Pages', 'yapb'), __('A single page shows a published post on its own.<br />You probably want to show the whole image -<br />But you can use thumbnailing here too, if you have design restrictions for example.', 'yapb'),
							array(
								'yapb_display_images_single' => array('CHECKBOX', __('<strong>Display images on SINGLE pages.</strong>', 'yapb'), true),
								'yapb_display_images_single_thumbnail' => array('CHECKBOX_INPUT', __('Display as thumbnail with a width of #3 px', 'yapb'), array(true, '460')),
								'yapb_display_images_single_inline_style' => array('INPUT', __('Inline CSS-Style for image tag: #40', 'yapb'), ''),
								'yapb_display_images_single_html_before' => array('TEXTAREA', __('Custom HTML before image tag', 'yapb'), '<div style="margin-bottom:20px;">'),
								'yapb_display_images_single_html_after' => array('TEXTAREA', __('Custom HTML after image tag', 'yapb'), '</div>')
							)
						),
						array(__('Archive Pages', 'yapb'), __('Archive pages usually show an overview of all published posts in a category, date range, etc.<br />You probably want to use thumbnails here.', 'yapb'),
							array(
								'yapb_display_images_archive' => array('CHECKBOX', __('<strong>Display images on ARCHIVE overview page listings.</strong>', 'yapb'), true),
								'yapb_display_images_archive_thumbnail' => array('CHECKBOX_INPUT', __('Display as thumbnail with a width of #3 px', 'yapb'), array(true, '100')),
								'yapb_display_images_archive_inline_style' => array('INPUT', __('Inline CSS-Style for image tag: #40', 'yapb'), ''),
								'yapb_display_images_archive_html_before' => array('TEXTAREA', __('Custom HTML before image tag', 'yapb'), '<div style="float:left;border:10px solid silver;margin-right:10px;margin-bottom:10px;">'),
								'yapb_display_images_archive_html_after' => array('TEXTAREA', __('Custom HTML after image tag', 'yapb'), '</div>')
							)
						),
						array(__('Content Pages', 'yapb'), __('You may post images to your content pages if you activate the according option above.<br />On content pages you probably want to show the original image.', 'yapb'),
							array(
								'yapb_display_images_page' => array('CHECKBOX', __('<strong>Display images on CONTENT pages.</strong>', 'yapb'), true),
								'yapb_display_images_page_thumbnail' => array('CHECKBOX_INPUT', __('Display as thumbnail with a width of #3 px', 'yapb'), array(false, '100')),
								'yapb_display_images_page_inline_style' => array('INPUT', __('Inline CSS-Style for image tag: #40', 'yapb'), ''),
								'yapb_display_images_page_html_before' => array('TEXTAREA', __('Custom HTML before image tag', 'yapb'), '<div style="float:left;border:10px solid silver;margin-right:10px;margin-bottom:10px;">'),
								'yapb_display_images_page_html_after' => array('TEXTAREA', __('Custom HTML after image tag', 'yapb'), '</div>')
							)
						)
					)
				)
			);
			
		}

		/**
		 * This method sets the plugins version information with the 
		 * values fetched from the readme.txt file in the plugins root directory
		 **/
		function readVersionInformation() {

			// Get the readme.txt file contents
			$readmeContent = file_get_contents(realpath(dirname(__FILE__) . '/../readme.txt'));
			
			// Since we don't want to execute regular expressions against the whole file, we extract the header part
			$readmeHeader = substr($readmeContent, 0, strpos($readmeContent, '== Description =='));
			
			// Extract the defined version numbers
			$this->pluginVersion = $this->_parseReadmeValue($readmeHeader, 'Stable tag');
			$this->requiredWordPressVersion = $this->_parseReadmeValue($readmeHeader, 'Requires at least');
			$this->highestTestedWordPressVersion = $this->_parseReadmeValue($readmeHeader, 'Tested up to');
		
		}

			/** 
			 * Function extracts a value from the readme header
			 *
			 * @param string $readmeHeader
			 * @param string $key
			 * @return string
			 **/
			function _parseReadmeValue($readmeHeader, $key) {
				preg_match('#' . $key . '\s*:\s*([0-9.]+)#i', $readmeHeader, $match);
				return $match[1];
			}

		/**
		 * POST.PHP
		 **/

		/**
		 * post.php hook edit_form_advanced
		 * This method enhances the wp new/edit-form
		 * to an upload form
		 **/
		function edit_form_advanced() {
			
			global $post;

			// Let's have a look if this post has an image attached
			// If yes: Assign the image to the template
			if (isset($post->ID)) {
				if (!is_null($image = YapbImage::getInstanceFromDb($post->ID))) {
					$this->tpl->assign('image', $image);
				}
			}

			$this->tpl->assign('content', $this->tpl->fetch('edit_form_advanced_field_fileupload.tpl.php'));
			$this->tpl->display('edit_form_advanced_javascript_injection.tpl.php');

		}

		/**
		 * This internal method returns if the given mime type gets
		 * accepted by YAPB upload
		 * @param string $type 
		 */
		function _isAllowedMimeType($type) {
			return in_array($type, array('image/jpeg', 'image/jpg', 'image/png', 'image/gif'));
		}

		/**
		 * This method hooks into the default upload workflow and intercepts 
		 * eventually existing file uploads
		 * 
		 * hook edit_post
		 * hook publish_post
		 * hook save_post
		 * @param number $post_id
		 **/
		function edit_publish_save_post($post_id) {

			global $wpdb;

			// if we have an yapb-imageupload here
			if (array_key_exists('yapb_imageupload', $_FILES)) {

				// wp_handle_upload: admin_functions.php
				$uploadedFileInfo = wp_handle_upload($_FILES['yapb_imageupload'], array('action' => $_POST['action']));

				// if we didn't have errors while upload
				if (!isset($uploadedFileInfo['error'])) {

					$url = $uploadedFileInfo['url'];
					$type = $uploadedFileInfo['type'];

					// We want to save the relative URI seen from the webhost-root of the image
					// We now have an url like 
					// http://my.server.tld/blog/wp-content/uploads/.../bla.jpg or
					// http://my.server.tld/wp-content/uploads/.../bla.jpg
					// We want to save instead: 
					// /blog/wp-content/uploads/.../bla.jpg or respectivly
					// /wp-content/uploads/.../bla.jpg

					$siteUrl = get_option('siteurl');
					if (substr($siteUrl, -1) != '/') $siteUrl .= '/';
					$uri = substr($url, strpos($siteUrl, '/', strpos($url, '//')+2));

					if ($this->_isAllowedMimeType($type)) {

						// First we take a look if we already have an image attached to this post
						// In this case we delete it, because we replace images

						if ($post_id) {
							if (!is_null($image = YapbImage::getInstanceFromDb($post_id))) {
								$image->delete();
							}
						}

						$image = new YapbImage(null, $post_id, $uri);

						// We persist the image to the database
						$image->persist();

						// Since we want to learn from every single posted image
						// Use YapbImage for Exif-Extraction
						$yapbImage = new YapbImage($id, $post_id, $uri); 
						$exifData = ExifUtils::getExifData($yapbImage, true);
						ExifUtils::learnTagnames($exifData);

						// If the user wants the postdate to be overwritten by the exif datetime stamp
						if (array_key_exists('exifdate', $_POST)) {

							if (!is_null($exifData)) {
								if (array_key_exists('DateTime', $exifData)) {

									// The exif date is formated this way: yyyy:mm:dd hh:mm:ss
									// strtotime needs it this way: yyyy-mm-dd hh:mm:ss
									// so we change that with a little regex since sprintf
									// isn't available on every platform

									$datetime = preg_replace('#([0-9]{4}):([0-9]{2}):([0-9]{2})#', '$1-$2-$3', $exifData['DateTime']);
									$date = strtotime($datetime);
									$dateGMT = $date - (get_option('gmt_offset') * 60 * 60);
									
									// we update the post if the datetime parsing was successful

									if ($date != -1) {

										// now we update post_date and post_date_gmt
										$wpdb->query('UPDATE ' . $wpdb->posts . ' set post_date = \'' . strftime('%Y-%m-%d %H:%M:%S', $date) . '\', post_date_gmt = \'' . strftime('%Y-%m-%d %H:%M:%S', $dateGMT) . '\' WHERE ID = ' . $post_id);

									} 

								} 
							}
							
						}

						// TODOs: 
						// - EXIF migration from source to target
						// - Testing

						// Optional image resize on upload
						
						if (get_option('yapb_resize_on_upload_activate')) {

							$maxSideLength = get_option('yapb_resize_on_upload_max_dimension');
							$width = $yapbImage->width;
							$height = $yapbImage->height;

							if ($width > $maxSideLength) {

								if ($height > $width) $transformResult = $yapbImage->transform(array('h=' . $maxSideLength));
								else $transformResult = $yapbImage->transform(array('w=' . $maxSideLength));
								$yapbImage->_fetchImagesize();

							} else 

							if ($height > $maxSideLength) {

								$transformResult = $yapbImage->transform(array('h=' . $maxSideLength));
								$yapbImage->_fetchImagesize();

							}

						}

						// TODO: Give the user feedback if the transformation fails
						// = Have a look how to manipulate the wp-post-success-message over a hook
						// = Catch other errors and give feedback too

					} else {

						// This ain't an image - let's delete it right away
						unlink(realpath($uri));

					}

				} else {
					
					// Some error occured while uploading the file
					// TODO: Some kind of error message on the admin interface

				}

			}

			if (array_key_exists('yapb_remove_image', $_POST)) {
				if (!is_null($image = YapbImage::getInstanceFromDb($post_id))) {
					$image->delete();
				}
			}

		}

		/**
		 * This hook reacts on a publish and pings all additionally 
		 * defined sites IF an image was attached
		 * @param number $post_id
		 */
		function publish_post($post_id) {
			
			// Since we registered this action after edit_publish_save_post($post_id),
			// we should eventually have an image 
			if (!is_null($image = YapbImage::getInstanceFromDb($post_id))) {
				// If this is a photoblog post, we additionally ping all sites defined by the user
				$this->_generic_yapb_ping($post_id);
			}

		}

		/**
		 * hook delete_post (wp-admin/post.php)
		 * This method gets called every time we delete a post
		 * @param number $post_id
		 **/
		function delete_post($post_id) {
			if (!is_null($image = YapbImage::getInstanceFromDb($post_id))) {
				$image->delete();
			}
		}

		/**
		 * EDIT.PHP
		 **/

		/**
		 * hook manage_posts_columns
		 * i want to insert another column after the id column
		 * since this is an associative array i've to rebuild it
		 * @param array $post_columns
		 **/
		function manage_posts_columns($posts_columns) {
			$result = array();
			foreach ($posts_columns as $key => $value) {
				if ($key == 'id') {
					$result[$key] = $value;
					$result['thumb'] = __('Thumbnails', 'yapb');
				} else $result[$key] = $value;
			}
			return $result;
		}

		/**
		 * this filter acts on the previously inserted post column 'thumb'
		 * see method manage_posts_columns
		 * @param string $column_name
		 **/
		function manage_posts_custom_column($column_name) {
			if ($column_name == 'thumb') {
				global $post;
				if (!is_null($image = YapbImage::getInstanceFromDb($post->ID))) {
					$this->tpl->assign('image', $image);
				} else {
					$this->tpl->clear('image');
				}
				$this->tpl->display('manage_posts_custom_column.tpl.php');
			}
		}

		//
		// The wordpress-loop
		// Goal: Insert an image-object into every post having attached one
		//

		/**
		 * filter for hook: the_posts
		 * We cycle through all posts, look for attached images
		 * and assign found images to the particular posts
		 * @param array $posts
		 */
		function the_posts(&$posts) {
			for ($i=0, $len=count($posts); $i<$len; $i++) {
				$post = &$posts[$i];
				if (!is_null($image = YapbImage::getInstanceFromDb($post->ID))) {
					$post->image = $image;
				}
			}
			return $posts;
		}

		/**
		 * Options page
		 * We insert an options page for yapb
		 **/
		function add_options_panel() {
			if (function_exists('add_options_page')) {
				// We need some libraries on this page
				wp_enqueue_script('prototype');
				wp_enqueue_script('dbx');
				add_options_page('Yapb', 'YAPB', 8, basename(__FILE__), array(&$this, 'render_options_panel_content'));
		    }
		}

		function render_options_panel_content() {
			$this->tpl->assign('options', $this->options);
			$this->tpl->assign('yapbVersion', $this->pluginVersion);
			$this->tpl->display('admin_panel_yapb_options_page.tpl.php');
		}

		//
		// Installation/Activation
		//

		/**
		 * Action for hook: activate_pluginurl
		 * Creates the needed yabp-databaes-table upon activation of the plugin
		 */
		function activate_pluginurl() {

			global $wpdb;

			if($wpdb->get_var('show tables like "' . YAPB_TABLE_NAME . '"') != YAPB_TABLE_NAME) {

				$sql = 'CREATE TABLE ' . YAPB_TABLE_NAME . ' ( ' .
					'id BIGINT NOT NULL AUTO_INCREMENT, ' .
					'post_id BIGINT NOT NULL, ' .
					'URI VARCHAR(255) NOT NULL, ' .
					'PRIMARY KEY (id), ' .
					'INDEX idx_01(post_id) ' .
				');';

				require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
				dbDelta($sql);

			}

			// YAPB Options

			$this->_create_options($this->options);

		}

		/**
		 * This hook gets called on deactivation of YAPB
		 */
		function deactivate_pluginurl() {
			
			// Reviewed 08.02.2007: 
			// Don't think it's neccessary but i'll leave this 
			// code fragment in the class so i have it if i need it...
			// Hey: I'm over 30 and tend to forget hook names from time to time ;-)
			
		}

		/**
		 * this hook gets fired at the bottom of the "Latest Activity" box on the WordPress
		 **/
		function activity_box_end() {

			require_once realpath(dirname(__file__) . '/YapbMaintainance.class.php');
			$this->tpl->assign('yapbMaintainanceInstance', new YapbMaintainance());
			$this->tpl->assign('pluginVersion', $this->pluginVersion);
			$this->tpl->display('dashboard_activity_div.tpl.php');

		}

		/** 
		 * This method enhances the various feeds for an image tag if available
		 * 
		 * @param string $content the content of the post
		 **/ 
		function the_content($content) {
			
			global $post;
			$result = $content;
			
			// We only have to alter the content if we have an image
			if ($post->image) {
			
				// Was this hook called out of a feed generation?
				if (is_feed()) {

					// Does the user want to display images in feeds?
					if (get_option('yapb_display_images_xml')) {
						
						// Please notice: WordPress feeds embed content in CDATA fields
						// So i definitly don't want to use xhtml in these content fields
						// Example: VFXY.com doesn't like &amp; in these URL's

						// Build the image tag
						
						$embed = get_option('yapb_display_images_xml_html_before') . '<img src="';
						
						if (get_option('yapb_display_images_xml_thumbnail_activate')) {
							$options = array('w=' . get_option('yapb_display_images_xml_thumbnail'));
							$embed .= 
								$post->image->getThumbnailHref(
									$options, 
									false // I manually override the users xhtml setting for WordPress feeds: Have a look above
								);
							$embed .= '" ';
							$imageWidth = $post->image->getThumbnailWidth($options);
							$imageHeight = $post->image->getThumbnailHeight($options);
						} else {
							$embed .= $post->image->getFullHref() . '" ';
							$imageWidth = $post->image->width;
							$imageHeight = $post->image->height; 
						}
						
						$embed .= 'width="' . $imageWidth . '" ';
						$embed .= 'height="' . $imageHeight . '" ';
						
						$style = get_option('yapb_display_images_xml_inline_style');
						if ((!is_null($style)) && ($style != '')) {
							$embed .= 'style="' . $style . '" ';
						}
						$embed .= '>' . get_option('yapb_display_images_xml_html_after');
						
						// Surround the image tag with a link
						// Thanks to fsimo for the idea

						$embed = '<a href="' . get_permalink() . '">' . $embed . '</a>';
						
						// Directly print out the image tag into 
						// the feed
						
						print $embed;
						
					}
					
				} else
				
				// If automatic image insertion is activated in general
				if (get_option('yapb_display_images_activate')) {
					print $this->_getImageTag($post->image);
				}
				
			} 
			
			return $result;
			
		}

			/**
			 * Internal method prints an image tag eventually surrounded
			 * by some html at the beginning of a posts content part
			 * 
			 * @param YapbImage $image
			 * @return string the complete image tag
			 */
			function _getImageTag($image) {
				
				global $post;
				
				$result = '<!-- no image -->';
				
				// Accepted areas of the blog
				$areas = array(
					array('home', is_home()),			// homepage
					array('single', is_single()),		// single page
					array('archive', is_archive()),		// archive page
					array('page', is_page())			// page
				);
				
				// print image into the post content according to the area-setting
				foreach ($areas as $area) {
					
					// If automatic image insertion is activated for this area
					if (get_option('yapb_display_images_' . $area[0]) && $area[1]) {
						
						// Does the user wants straight xhtml-href's?
						$xhtml = get_option('yapb_display_images_xhtml');
						
						// Does the user wants thumbnails?
						if (get_option('yapb_display_images_' . $area[0] . '_thumbnail_activate')) {
							$options = array('w=' . get_option('yapb_display_images_' . $area[0] . '_thumbnail'));
							$imageURI = $image->getThumbnailHref($options);
							$imageWidth = $image->getThumbnailWidth($options);
							$imageHeight = $image->getThumbnailHeight($options);
						} else {
							$imageURI = $image->uri;
							$imageWidth = $image->width;
							$imageHeight = $image->height; 
						}
						
						// Did the user define an inline style definition?
						$style = get_option('yapb_display_images_' . $area[0] . '_inline_style');
						if ((!is_null($style)) && ($style != '')) {
							$style = ' style="' . $style . '"';
						}
						
						$result = get_option('yapb_display_images_' . $area[0] . '_html_before');
						$result .= '<img' . $style . ' width="' . $imageWidth . '" height="' . $imageHeight . '" src="' . $imageURI . '" title="' . $post->post_title . '" alt="' . $post->post_title . '"' . ($xhtml ? ' />' : '>');
						$result .= get_option('yapb_display_images_' . $area[0] . '_html_after');
						
					}
					
				}
				
				return $result;				
				
			}
			
		
		/**
		 * This method additionally pings all sites provided by the user if he
		 * publishes a photoblog article.
		 * It's a 1:1 copy of the wp-function found in wp-includes/functions.php
		 * The only modification: it uses the option yapb_ping_sites instead of 
		 * ping_sites defined in options/write
		 * 
		 * @param number $post_id
		 **/
		function _generic_yapb_ping($post_id = 0) {
			$services = get_settings('yapb_ping_sites');
			$services = preg_replace("|(\s)+|", '$1', $services); // Kill dupe lines
			$services = trim($services);
			if ( '' != $services ) {
				$services = explode("\n", $services);
				foreach ($services as $service) {
					weblog_ping($service);
				}
			}
			return $post_id;
		}

		/**
		 * This method is a small adaption of the function wp_dropdown_cats(...) 
		 * to be found in /wp-admin/admin-functions.php
		 * 
		 * 
		 * @param number $currentcat
		 * @param number $currentparent
		 * @param number $parent
		 * @param number $level
		 * @param array $caegories
		 */
		function _options_categories_array($currentcat = 0, $currentparent = 0, $parent = 0, $level = 0, $categories = 0) {

			global $wpdb;
			$result = array();

			if (!$categories) {
				$categories = get_categories( 'hide_empty=0' );
			}

			if ($categories) {
				foreach ($categories as $category) {
					if ($currentcat != $category->term_id && $parent == $category->parent) {
						$pad = str_repeat(' ', $level);
						$category->name = wp_specialchars($category->name);
						$result[$pad . $category->name] = $category->term_id;
						$result = $result + $this->_options_categories_array($currentcat, $currentparent, $category->term_id, $level+1, $categories);
					}
				}
			}

			return $result;

		}

		/**
		 * This method creates all options needed by YAPB and fills them with their default values
		 * @param array $optionsArray the YAPB options array 
		 **/
		function _create_options($optionsArray) {
			foreach ($optionsArray as $optionGroup) {
				foreach($optionGroup[2] as $optionSubGroup) {
					foreach ($optionSubGroup[2] as $optionName => $optionArray) {
						
						$optionType = $optionArray[0];
						$optionText = $optionArray[1];
						$optionDefaultValue = $optionArray[2];

						switch ($optionType) {

							case 'CHECKBOX_INPUT' :

								add_option($optionName . '_activate', $optionDefaultValue[0]);
								add_option($optionName, $optionDefaultValue[1]);
								break;

							case 'CHECKBOX_SELECT' :

								$optionDefaultValue = $optionArray[3];
								add_option($optionName . '_activate', $optionDefaultValue[0]);
								add_option($optionName, $optionDefaultValue[1]);
								break;

							case 'SELECT' :

								$optionDefaultValue = $optionArray[3];
								add_option($optionName, $optionDefaultValue);
								break;

							case 'CHECKBOX' :
							case 'INPUT' :
							case 'TEXTAREA' :

								add_option($optionName, $optionDefaultValue); 
								break;
							
							case 'CUSTOM_VIEW_EXIF_TAGNAMES' :
								
								add_option($optionName, '');
								break;

						}

					}

				}

			}

		}

	}

?>
